@@ -16,7 +16,7 @@ The description of a referenced resource, and the descriptions of the associated
 
 The ContextObject may contain up to six Entities. One of these, the Referent, conveys information about the referenced item. It must always be included in a ContextObject. The other five entities - ReferringEntity, Requester, Resolver, ServiceType and Referrer - hold information about the context of the reference and are optional.
 
-=head1 OpenURL
+= OpenURL
 
 http://library.caltech.edu/openurl/
 
@@ -65,7 +65,7 @@ There are currently two formats for ContextObjects defined in the OpenURL Framew
 		)->as_string();
 
 	# Parsing (wrappers for $uri->query_form())
-	my $uri = URI::OpenURL->new('http://a.OpenURL/?url_ver=Z39.88-2004&...');
+	my $uri = URI::OpenURL->new('http://a.openurl/?url_ver=Z39.88-2004&...');
 	my @referent = $uri->referent->metadata();
 	print join(',',@referent), "\n";
 	# This could lose data if there is more than one id
@@ -76,10 +76,6 @@ There are currently two formats for ContextObjects defined in the OpenURL Framew
 		my %md = $uri->referent->metadata();
 		print ($md{genre} || 'Unknown journal article genre'), "\n";
 	}
-	
-	if( $uri->referent->val_fmt() eq 'info:ofi/fmt:kev:mtx:journal' ) {
-		print "The referent is a journal article.\n";
-	}
 
 =head1 METHODS
 
@@ -89,30 +85,22 @@ There are currently two formats for ContextObjects defined in the OpenURL Framew
 
 use vars qw( $VERSION );
 
-$VERSION = '0.4.6';
+$VERSION = '0.2';
 
 use strict;
 use URI::Escape;
 use Carp;
 use POSIX qw/ strftime /;
 
-require URI;
 require URI::_server;
 use vars qw( @ISA );
-@ISA = qw( URI::_server );
+@ISA = qw( URI::http );
 
 =pod
 
-=item $uri = URI::OpenURL->new([$url])
+=item $uri = URI::openurl->new([$url])
 
-Create a new URI::OpenURL object and optionally initialize with $url. If $url does not contain a query component (...?key=value) the object will be initialized to a valid contextobject, but without any entities.
-
-If you don't want the context object version and encoding specify url_ver e.g.
-
-	use URI::OpenURL;
-	my $uri = URI::OpenURL->new(
-		'http://myresolver.com/openURL?url_ver=Z39.88-2004'
-	);
+Create a new URI::openurl object and optionally initialize with $url. If $url does not contain a query component (...?key=value) the object will be initialized to a valid contextobject, but without any entities.
 
 =cut
 
@@ -123,225 +111,15 @@ sub new {
 sub _init {
 	my $self = shift->SUPER::_init(@_);
 	$self->query_form(
-		url_ver => 'Z39.88-2004',
-	) unless $self->query();
-	$self;
-}
-
-=pod
-
-=item $uri = URI::OpenURL->new_from_hybrid($uri)
-
-Create a new URI::OpenURL object from a hybrid OpenURL (version 0.1 and/or 1.0 KEVS). Use this to parse a version 0.1 (SFX) style OpenURL.
-
-=cut
-
-sub new_from_hybrid
-{
-	my ($class,$uri) = @_;
-	$class = ref($class) || $class;
-	my $self = $class->new($uri);
-	# If we already have a 1.0 OpenURL just return a canonical 1.0
-	my @KEVS = $self->query_form;
-	my %kevs = @KEVS;
-	my $genre = $kevs{'genre'} || 'article';
-	if( $kevs{url_ver} && $kevs{url_ver} eq 'Z39.88-2004' ) {
-		return $self->canonical();
-	}
-	# Initialize the OpenURL (url_ver etc.)
-	$self->query('');
-	$self = $class->new($self);
-	# Tidy up the 0.1 keys
-	for(my $i = 0; $i < @KEVS;) {
-		if( $KEVS[$i] eq 'sid' ) {
-			$self->referrer->id('info:sid/'.$KEVS[$i+1]);
-			splice(@KEVS,$i,2);
-		} elsif( $KEVS[$i] eq 'id' ) {
-			if( $KEVS[$i+1] =~ s/(^doi|pmid|bibcode):// ) {
-				$self->referent->id("info:$1/".$KEVS[$i+1]);
-			} else {
-				$self->referent->id($KEVS[$i+1]);
-			}
-			splice(@KEVS,$i,2);
-		} elsif( $KEVS[$i] eq 'pid' ) {
-			$self->referent->dat($KEVS[$i+1]);
-			splice(@KEVS,$i,2);
-		} else {
-			$KEVS[$i] = 'jtitle' if $KEVS[$i] eq 'title';
-			$i += 2;
-		}
-	}
-	# Map genre onto a ctx format and add the metadata
-	if( $genre =~ '^article|preprint|proceeding$' ) {
-		$self->referent->journal(@KEVS);
-	} elsif( $genre eq 'bookitem' ) {
-		$self->referent->book(@KEVS);
-	} else {
-		die "Unable to handle version 0.1 genre: $genre";
-	}
-	$self;
-}
-
-=pod
-
-=item @qry = $uri->query_form([key, value, [key, value]])
-
-Equivalent to URI::query_form, but with support for UTF-8 encoding.
-
-=cut
-
-sub query_form
-{
-	my $self = shift;
-	my @new = @_;
-	if( 1 == @new ) {
-		my $n = $new[0];
-		if( ref($n) eq "ARRAY" ) {
-			@new = @$n;
-		} elsif( ref($n) eq "HASH" ) {
-			@new = %$n;
-		}
-	}
-	for (@new) {
-		utf8::encode($_);
-	}
-	map { utf8::decode($_); $_ } $self->SUPER::query_form(@new);
-}
-
-=pod
-
-=item $uri->init_ctxobj_version()
-
-Add ContextObject versioning.
-
-=cut
-
-sub init_ctxobj_version
-{
-	my $self = shift;
-	my %query = $self->query_form;
-	return if
-		defined($query{'ctx_ver'}) &&
-		$query{'ctx_ver'} eq 'Z39.88-2004';
-	$self->query_form(
-		$self->query_form,
 		ctx_ver => 'Z39.88-2004',
 		ctx_enc => 'info:ofi/enc:UTF-8',
+		ctx_id => '1', # TODO Check whether ctx_id is significant for URIs
+		ctx_tim => strftime("%Y-%m-%dT%H:%M:%STZD",gmtime(time)),
+		url_ver => 'Z39.88-2004',
+		url_tim => strftime("%Y-%m-%dT%H:%M:%STZD",gmtime(time)),
 		url_ctx_fmt => 'info:ofi/fmt:kev:mtx:ctx',
-	);
-}
-
-=pod
-
-=item $ts = $uri->init_timestamps([ctx_timestamp, [url_timestamp]])
-
-Add ContextObject and URL timestamps, returns the old timestamp(s) or undef on none.
-
-=cut
-
-sub init_timestamps {
-	my $self = shift;
-	my $ctx_timestamp = shift ||
-		strftime("%Y-%m-%dT%H:%M:%STZD",gmtime(time));
-	my $url_timestamp = shift || $ctx_timestamp;
-	my @query = $self->query_form;
-	my @old;
-	for(my $i = 0; $i < @query;) {
-		if( $query[$i] eq 'ctx_tim' ) {
-			($_,$old[0]) = splice(@query,$i,2);
-		} elsif( $query[$i] eq 'url_tim' ) {
-			($_,$old[1]) = splice(@query,$i,2);
-		} else {
-			$i+=2;
-		}
-	}
-	$self->query_form(
-		@query,
-		'ctx_tim', $ctx_timestamp,
-		'url_tim', $url_timestamp,
-	);
-	wantarray ? @old : ($old[0]||$old[1]);
-}
-
-=pod
-
-=item $uri = $uri->as_hybrid()
-
-Return the OpenURL as a hybrid 0.1/1.0 OpenURL (contains KEVs for both versions). Returns a new URI::OpenURL object.
-
-=cut
-
-sub as_hybrid
-{
-	my $self = shift;
-	my @KEVS = $self->query_form;
-	# Add the referent
-	my @md = $self->referent->metadata();
-	# 'title' has been changed to 'jtitle' in 1.0
-	for(my $i = 0; $i < @md; $i+=2) {
-		$md[$i] = 'title' if($md[$i] eq 'jtitle');
-	}
-	push @KEVS, @md;
-	# Add the referrer's id
-	my $rfr_id = $self->referrer->id;
-	if( defined($rfr_id) && $rfr_id =~ s/^info:sid\/// ) {
-		push @KEVS,	sid => $rfr_id;
-	}
-	# Add the referent's id (if its compatible with 0.1)
-	my $rft_id = $self->referent->id;
-	if( defined($rft_id) &&
-		($rft_id =~ s/^info:(doi|pmid|bibcode)\//$1:/ ||
-		 $rft_id =~ /^oai:/)
-	) {
-		push @KEVS,	id => $rft_id;
-	}
-	# Return a new URI (otherwise we pollute ourselves)
-	my $hybrid = new URI::OpenURL($self);
-	$hybrid->query_form(@KEVS);
-	$hybrid;
-}
-
-=item $uri = $uri->canonical()
-
-Return a canonical OpenURL by removing anything that isn't part of the version 1.0 specification.
-
-=cut
-
-sub canonical
-{
-	my $uri = shift->SUPER::canonical();
-	$uri = bless $uri, "URI::OpenURL";
-	my @KEVS = $uri->query_form();
-	for(my $i = 0; $i < @KEVS; ) {
-		if( $KEVS[$i] !~ /^ctx_ver|ctx_enc|ctx_id|ctx_tim|url_ver|url_tim|url_ctx_fmt|(?:(?:rft|rfe|svc|req|res|rfr)[_\.].+)$/ ) {
-			splice(@KEVS,$i,2);
-		} else {
-			$i += 2;
-		}
-	}
-	$uri->query_form(@KEVS);
-	$uri;
-}
-
-=pod
-
-=item $str = $uri->dump()
-
-Return the OpenURL as a human-readable string (useful for debugging).
-
-=cut
-
-sub dump
-{
-	my $self = shift;
-	my $str = URI->new($self);
-	$str->query('');
-	$str .= "\n";
-	my @kevs = $self->query_form;
-	for(my $i = 0; $i < @kevs; $i+=2) {
-		$str .= $kevs[$i] . "=" . $kevs[$i+1] . "\n";
-	}
-	$str;
+	) unless $self->query_form();
+	$self;
 }
 
 =pod
@@ -353,7 +131,7 @@ Every ContextObject must have a Referent, the referenced resource for which the
 =cut
 
 sub referent {
-	my $self = bless shift, 'URI::OpenURL::referent';
+	my $self = bless shift, 'OpenURL::referent';
 	return $self->descriptors() if wantarray;
 	$self->_addattr(@_);
 }
@@ -367,7 +145,7 @@ The ReferringEntity is the Entity that references the Referent. It is optional i
 =cut
 
 sub referringEntity {
-	my $self = bless shift, 'URI::OpenURL::referringEntity';
+	my $self = bless shift, 'OpenURL::referringEntity';
 	return $self->descriptors() if wantarray;
 	$self->_addattr(@_);
 }
@@ -381,7 +159,7 @@ The Requester is the Entity that requests services pertaining to the Referent. I
 =cut
 
 sub requester {
-	my $self = bless shift, 'URI::OpenURL::requester';
+	my $self = bless shift, 'OpenURL::requester';
 	return $self->descriptors() if wantarray;
 	$self->_addattr(@_);
 }
@@ -393,7 +171,7 @@ The ServiceType is the Entity that defines the type of service requested. It is
 =cut
 
 sub serviceType {
-	my $self = bless shift, 'URI::OpenURL::serviceType';
+	my $self = bless shift, 'OpenURL::serviceType';
 	return $self->descriptors() if wantarray;
 	$self->_addattr(@_);
 }
@@ -407,7 +185,7 @@ The Resolver is the Entity at which a request for services is targeted. It is op
 =cut
 
 sub resolver {
-	my $self = bless shift, 'URI::OpenURL::resolver';
+	my $self = bless shift, 'OpenURL::resolver';
 	return $self->descriptors() if wantarray;
 	$self->_addattr(@_);
 }
@@ -421,7 +199,7 @@ The Referrer is the Entity that generated the ContextObject. It is optional in t
 =cut
 
 sub referrer {
-	my $self = bless shift, 'URI::OpenURL::referrer';
+	my $self = bless shift, 'OpenURL::referrer';
 	return $self->descriptors() if wantarray;
 	$self->_addattr(@_);
 }
@@ -454,36 +232,21 @@ Returns by-value metadata as a list of key-value pairs for the current entity (r
 
 Optionally, if you wish to add metadata that does not use one of the standard schemas (journal, book etc.) then you can add them using metadata.
 
-=item @vals = $uri->referent->descriptor('id')
+=head1 ABOUT
 
-Return a list of values given for an entity descriptor (id, ref, dat, val_fmt, ref_fmt).
+This module should be considered BETA, not least because the OpenURL standard is (as of 2004-05-12) "submitted to NISO for Ballot". However it is not intended to change the interface.
 
-=item $dat = $uri->referent->dat()
+=head1 TODO
 
-=item @ids = $uri->referent->id()
+"Easy" access to descriptors and metadata, e.g. $uri->referent->descriptor->id().
 
-=item $ref = $uri->referent->ref()
+=head1 NOTES
 
-=item $val_fmt = $uri->referent->val_fmt()
+This module works a little differently to the other URI modules in that structured data is added by calling methods that return an object. Whenever an entity method is called the object gets casted into a class encapsulating that entity, allowing further method calls to add metadata. The net result of this is that the URI object will change type as you add data to the object.
 
-=item $ref_fmt = $uri->referent->ref_fmt()
+The reason for doing this is URI only allows one value to be stored (the URI string), so to keep the context of which entity the user is adding data to the only thing I could think of was to change the class. The downside of this is you can't use URI::openurl as a superclass.
 
-Return the respective descriptor using a method interface. An entity may contain 0 or more ids, and optionally a by-reference URI, private data, by-value format and by-reference format.
-
-=head1 CHANGES
-
-	0.4.6
-		- Removed ContextObject versioning from default
-		  constructor
-	0.4.5
-		- Support for URL utf-8 encoding
-	0.4.2
-		- Added methods for parsing/writing hybrid OpenURLs
-	0.4.1
-		- Timestamps no longer included in default initialization
-		- Added method "init_timestamps" to add timestamps
-	0.4
-		- Initial release
+You will also notice that if you try creating an URI object with an OpenURL (which is actually http:) the http module will be called rather than openurl. You can either call openurl directly as given in the synopsis, or by changing 'http' to 'openurl', in which case URI will call the correct module.
 
 =head1 COPYRIGHT
 
@@ -503,7 +266,7 @@ University of Southampton, UK
 
 =cut
 
-package URI::OpenURL::entity;
+package OpenURL::entity;
 
 use vars qw(@ISA);
 @ISA = qw(URI::OpenURL);
@@ -530,8 +293,7 @@ sub _addattr {
 	my @KEVS = $self->query_form();
 	my $entity = _entity($self);
 	for( my $i = 0; $i < @pairs; $i+=2 ) {
-		next unless defined($pairs[$i+1]) && length($pairs[$i+1]);
-		push @KEVS, $entity.'_'.$pairs[$i], $pairs[$i+1];
+		push @KEVS, $entity.'_'.$pairs[$i], ($pairs[$i+1]||'');
 	}
 	$self->query_form(@KEVS);
 	$self;
@@ -543,11 +305,22 @@ sub _addkevs {
 	my $entity = _entity($self);
 	push @KEVS, $entity.'_val_fmt', $val_fmt if $val_fmt;
 	for( my $i = 0; $i < @pairs; $i+=2 ) {
-		next unless defined($pairs[$i+1]) && length($pairs[$i+1]);
-		push @KEVS, $entity.'.'.$pairs[$i], $pairs[$i+1];
+		#	if( $pairs[$i] =~ /^id|val_fmt|ref_fmt|ref|dat$/ ) {
+		push @KEVS, $entity.'.'.$pairs[$i], ($pairs[$i+1]||'');
 	}
 	$self->query_form(@KEVS);
-	bless $self, 'URI::OpenURL'; # Should catch some broken user code
+	bless $self, 'URI::openurl'; # Should catch some broken user code
+}
+
+sub descriptor {
+	my ($self,$key) = @_;
+	my @KEVS = $self->query_form();
+	my @VALS;
+	my $entity = $self->_entity();
+	for(my $i = 0; $i < @KEVS; $i+=2) {
+		push @VALS, $KEVS[$i+1] if( $KEVS[$i] eq "${entity}_${key}" );
+	}
+	@VALS;
 }
 
 sub dublinCore {
@@ -590,24 +363,6 @@ sub descriptors {
 	return @md;
 }
 
-sub descriptor {
-	my ($self,$key) = splice(@_,0,2);
-	$self->_addattr($key => $_) for @_;
-	my @KEVS = $self->query_form();
-	my @VALS;
-	my $entity = $self->_entity();
-	for(my $i = 0; $i < @KEVS; $i+=2) {
-		push @VALS, $KEVS[$i+1] if( $KEVS[$i] eq "${entity}_${key}" );
-	}
-	wantarray ? @VALS : $VALS[0];
-}
-
-sub dat { shift->descriptor('dat',@_) }
-sub id { shift->descriptor('id',@_) }
-sub ref { shift->descriptor('ref',@_) }
-sub ref_fmt { shift->descriptor('ref_fmt',@_) }
-sub val_fmt { shift->descriptor('val_fmt',@_) }
-
 # By-value metadata (things with '.' in)
 sub metadata {
 	my $self = shift;
@@ -624,35 +379,35 @@ sub metadata {
 	return @md;
 }
 
-package URI::OpenURL::referent;
+package OpenURL::referent;
 
 use vars qw(@ISA);
-@ISA = qw(URI::OpenURL::entity);
+@ISA = qw(OpenURL::entity);
 
-package URI::OpenURL::referringEntity;
+package OpenURL::referringEntity;
 
 use vars qw(@ISA);
-@ISA = qw(URI::OpenURL::entity);
+@ISA = qw(OpenURL::entity);
 
-package URI::OpenURL::requester;
+package OpenURL::requester;
 
 use vars qw(@ISA);
-@ISA = qw(URI::OpenURL::entity);
+@ISA = qw(OpenURL::entity);
 
-package URI::OpenURL::serviceType;
+package OpenURL::serviceType;
 
 use vars qw(@ISA);
-@ISA = qw(URI::OpenURL::entity);
+@ISA = qw(OpenURL::entity);
 
-package URI::OpenURL::resolver;
+package OpenURL::resolver;
 
 use vars qw(@ISA);
-@ISA = qw(URI::OpenURL::entity);
+@ISA = qw(OpenURL::entity);
 
-package URI::OpenURL::referrer;
+package OpenURL::referrer;
 
 use vars qw(@ISA);
-@ISA = qw(URI::OpenURL::entity);
+@ISA = qw(OpenURL::entity);
 
 1;
 
